/* MDB Tools - A library for reading MS Access database file
 * Copyright (C) 2000 Brian Bruns
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "mdbtools.h"

int kkd_dump_hdr(int fd, off_t offset);
int kkd_dump_row(int fd, off_t offset);

main(int argc, char **argv)
{
off_t offset=0;
int   fd, i;
char  xdigit;
short digit;
struct stat status;

	if (argc<2) {
		fprintf(stderr,"Usage: mdbdump <file> <offset>\n");
		fprintf(stderr,"     : offset is a hex offset to the KKD to dump\n");
		exit(1);
	}
	
	fd = mdb_open(argv[1],O_RDONLY);

	for (i=0;i<strlen(argv[2]);i++) {
		xdigit = argv[2][i];
		if (isalpha(xdigit)) digit = 10 + toupper(xdigit) - 'A';
		if (isdigit(xdigit)) digit = xdigit - '0';
		offset <<= 4;
		offset += digit;
	}

	fstat(fd, &status);
	if (status.st_size < offset) {
		fprintf(stderr,"offset %lu is beyond EOF\n",offset);
		close(fd);
		exit(1);
	}
	lseek(fd, offset, SEEK_SET);
	kkd_dump_hdr(fd, offset);
	kkd_dump_row(fd, offset);
	kkd_dump_row(fd, offset);
	kkd_dump_row(fd, offset);
	kkd_dump_row(fd, offset);
	kkd_dump_row(fd, offset);
}

int kkd_dump_hdr(int fd, off_t offset)
{
unsigned char  buf[BUFSIZ];
int   len, cur, flen;

	mdb_read(fd,buf,4);
	if (strncmp(buf,"KKD",4)) {
		fprintf(stderr,"offset %lu is not a KKD record\n",offset);
		close(fd);
		exit(1);
	}
	fprintf(stdout,"KKD record start:      %lu\n",offset);
	len = mdb_read_32(fd);
	fprintf(stdout,"KKD header size:       %lu\n",len);
	mdb_read(fd,buf,2);
	fprintf(stdout,"KKD hdr mystery bytes: %02x %02x\n",
		buf[0],buf[1]);
	cur = 6;   /* 4 (size) + 2 (mystery) */
	while (cur<len) {
		flen = mdb_read_16(fd);
		cur += 2;
		fprintf(stdout,"KKD hdr field length:  %lu\n",flen);
		cur += mdb_read(fd, buf, flen);
		buf[flen]='\0';
		fprintf(stdout,"KKD hdr field name:    %s\n",buf);
	}
}

int kkd_dump_row(int fd, off_t offset)
{
unsigned char  buf[BUFSIZ];
int   len;
	len = mdb_read_32(fd);
	fprintf(stdout,"KKD row size:          %lu\n",len);
	mdb_read(fd,buf,2);
	fprintf(stdout,"KKD row mystery bytes: %02x %02x\n",
		buf[0],buf[1]);
	mdb_read(fd,buf,len - 6); /* 4 (size field) + 2 (mystery bytes) */
}
