/* MDB Tools - A library for reading MS Access database files
 * Copyright (C) 2000 Brian Bruns
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "mdbtools.h"

int mdb_open(char *filename, int flags)
{
int fd;

	fd = open(filename,O_RDONLY);

	if (fd==-1) {
		fprintf(stderr,"Couldn't open file %s\n",filename);
		exit(1);
	}

	return fd;
}

/* 
** mdb_read a wrapper for read that bails if anything is wrong 
*/
size_t mdb_read(int fd, void *buf, size_t count)
{
size_t len;

	len = read(fd,buf,count);
	if (len==-1) {
		perror("read");
		exit(1);
	}
	else if (len<count) {
		fprintf(stderr,"EOF reached.\n");
		exit(1);
	}
	return len;
}
int mdb_read_16(int fd)
{
int   len;
unsigned char buf[2];

	/* all sizes in access are stored in little endian order */
	mdb_read(fd,buf,2);
	len = buf[1];
	len <<= 8;
	len += buf[0];

	return len;
}
long mdb_read_32(int fd)
{
long   len;
unsigned char buf[4];
int i;

	/* all sizes in access are stored in little endian order */
	mdb_read(fd,buf,4);
	len = buf[3];
	for (i=2;i>=0;i--) {
		len <<= 8;
		len += buf[i];
	}

	return len;
}
